"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BufferShader = exports.ShadertoyTexture = void 0;
const THREE = __importStar(require("three"));
const BASE_FRAG = `
precision highp float;
uniform vec2 iResolution;
uniform float iTime;
uniform int iFrame;
uniform vec4 iMouse;

varying vec2 vUv;
uniform sampler2D iChannel0;
uniform sampler2D iChannel1;
uniform sampler2D iChannel2;

void mainImage( out vec4,  vec2 fragCoord );
void main () {
    vec4 outfrag;
    mainImage(outfrag,iResolution*vUv);
    gl_FragColor = outfrag;
}
`;
const VERTEX_SHADER = `
    varying vec2 vUv;
    
    void main() {
        vUv = uv;
        gl_Position = vec4(position,1.0);
    }
`;
class ShadertoyTexture {
    set common(value) {
        this.bufferA.common = value;
        this.bufferB.common = value;
        this.bufferC.common = value;
        this.bufferD.common = value;
        this.image.common = value;
    }
    constructor(width, height) {
        this.width = width;
        this.height = height;
        this.uniforms = {
            iMouse: { value: new THREE.Vector4(0.5, 0.5, 0, 0) },
            iFrame: { value: 0 },
            iTime: { value: 0 },
            iResolution: { value: new THREE.Vector2(512, 512) },
        };
        this.clock = new THREE.Clock();
        this.bufferA = new BufferShader(this.width, this.height);
        this.bufferB = new BufferShader(this.width, this.height);
        this.bufferC = new BufferShader(this.width, this.height);
        this.bufferD = new BufferShader(this.width, this.height);
        this.image = new BufferShader(this.width, this.height);
        this.texture = this.image.outputBuffer.texture;
    }
    render(renderer) {
        this.uniforms.iTime.value = this.clock.getElapsedTime();
        this.uniforms.iFrame.value++;
        const renderOrder = [this.bufferA, this.bufferB, this.bufferC, this.bufferD, this.image];
        renderOrder.forEach((buffer) => {
            if (buffer.enabled) {
                // Update buffer uniforms with global uniforms:
                Object.keys(this.uniforms).forEach((key) => {
                    buffer.uniforms[key] = this.uniforms[key];
                });
                buffer.render(renderer);
            }
        });
        this.image.outputBuffer.texture.needsUpdate = true;
    }
}
exports.ShadertoyTexture = ShadertoyTexture;
function isTexture(input) {
    return input instanceof THREE.Texture;
}
const emptyTexture = new THREE.Texture();
class BufferShader {
    set shader(value) {
        this.enabled = true;
        const fragmentShader = BASE_FRAG + "\n" + this.common + "\n" + value;
        const material = new THREE.ShaderMaterial({
            fragmentShader,
            vertexShader: VERTEX_SHADER,
            uniforms: this.uniforms
        });
        this.plane.material = material;
    }
    constructor(width, height) {
        this.width = width;
        this.height = height;
        this.common = '';
        // TODO: Support non-texture channels, and do something more effecient here?
        this.iChannel = [emptyTexture, emptyTexture, emptyTexture, emptyTexture];
        this.uniforms = {
            iChannel0: { value: emptyTexture },
            iChannel1: { value: emptyTexture },
            iChannel2: { value: emptyTexture },
            iChannel3: { value: emptyTexture }
        };
        this.camera = new THREE.OrthographicCamera(-1, 1, 1, -1, -1, 1);
        this.enabled = false;
        this.outputBuffer = new THREE.WebGLRenderTarget(width, height, {
            minFilter: THREE.LinearFilter,
            magFilter: THREE.LinearFilter,
            format: THREE.RGBAFormat,
            encoding: THREE.LinearEncoding,
            type: THREE.FloatType,
            stencilBuffer: false,
        });
        this.readBuffer = this.outputBuffer.clone();
        this.scene = new THREE.Scene();
        this.scene.background = new THREE.Color(0x000000);
        this.plane = new THREE.Mesh(new THREE.PlaneGeometry(2, 2), new THREE.MeshBasicMaterial({ color: 0xff0000 }));
        this.scene.add(this.plane);
    }
    swap() {
        const temp = this.readBuffer;
        this.readBuffer = this.outputBuffer;
        this.outputBuffer = temp;
    }
    getIChannel(index) {
        const iChannel = this.iChannel[index];
        if (isTexture(iChannel))
            return iChannel;
        return iChannel.readBuffer.texture;
    }
    render(renderer) {
        // Update iChannels:
        this.uniforms.iChannel0.value = this.getIChannel(0);
        this.uniforms.iChannel1.value = this.getIChannel(0);
        this.uniforms.iChannel2.value = this.getIChannel(0);
        this.uniforms.iChannel3.value = this.getIChannel(0);
        renderer.setRenderTarget(this.outputBuffer);
        renderer.clear();
        renderer.render(this.scene, this.camera);
        renderer.setRenderTarget(null);
        this.swap();
    }
}
exports.BufferShader = BufferShader;
